==> prayer-1.0.12.tar.gz: prayer-1.0.12/prayer/cmd_vacation.c <==
/* $Cambridge: hermes/src/prayer/prayer/cmd_vacation.c,v 1.5 2004/04/21 13:11:21 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void generate_form(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct account *account = session->account;
    char *aliases = ((account->vacation_aliases)
                     ? (account->vacation_aliases) : NIL);
    char *days = string_itoa(request->pool, account->vacation_days);

    /* Title bar */
    html_start(session, b, "Vacation message");
    html_banner_toolbar(session, b, "vacation", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Vacation Message</h2>" CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "vacation");

    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    /* use_vacation */
    html_form_table_checkbox(b, "Enable Vacation Message.", "use_vacation",
                             account->vacation_enabled);

    if (session->sieved_server) {
        /* alt_addr */
        html_form_table_field(b, "&nbsp;&nbsp;&nbsp;Aliases:",
                              "aliases", aliases, 40);

        /* Days */
        html_form_table_field(b, "Days:", "days", days, 3);

    } else {
        /* alt_addr */
        html_form_table_field(b, "&nbsp;&nbsp;&nbsp;Aliases:",
                              "aliases", aliases, 50);
    }

    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);

    /* Start of vacation */
    bprintf(b,
            "<textarea name=\"vacation_msg\" rows=\"8\" cols=\"80\">"
            CRLF);
    if (account->vacation_msg && account->vacation_msg[0])
        html_quote_string(b, account->vacation_msg);
    bputs(b, "</textarea>" CRLF);

    /* Submit and cancel buttons */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    html_form_table_submit(b, "sub_apply", "Apply");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);

    /* End table */
    html_form_end(b);

    html_end(b);
    response_html(request, 200);        /* Success */
}

static void generate_error(struct session *session)
{
    struct request *request = session->request;
    struct account *account = session->account;
    char *msg = account_fetch_message(account);
    struct buffer *b = request->write_buffer;

    /* Title bar */
    html_start(session, b, "Vacation message");
    html_banner_toolbar(session, b, "vacation", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Vacation Messages</h2>" CRLF);

    bputs(b, "<b>Error:</b> ");
    if (msg)
        html_quote_string(b, msg);
    else
        bputs(b, "Unable to check mail processing status with server");
    bputs(b, "" CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "vacation");

    /* Cancel button */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    html_form_end(b);

    html_end(b);
    response_html(request, 200);
}

void cmd_vacation(struct session *session)
{
    struct request *request = session->request;
    struct assoc *h = NIL;
    struct account *account = session->account;
    ADDRESS *addr = NIL;
    ADDRESS *a = NIL;
    char *s;
    BOOL rc = T;

    if (request->method != POST) {
        if (!account_mail_check(account, request->pool))
            generate_error(session);
        else
            generate_form(session);
        return;
    }

    /* Form post */
    request_decode_post(request);
    h = request->form;

    if (!(s = assoc_lookup(h, "sub_apply"))) {
        session_redirect(session, request, "manage");   /* parent screen */
        return;
    }

    account->vacation_enabled =
        (assoc_lookup(h, "use_vacation")) ? T : NIL;

    if ((s = assoc_lookup(h, "vacation_msg")))
        string_strdup(&account->vacation_msg, s);
    else
        string_strdup(&account->vacation_msg, "");

    if ((s = assoc_lookup(h, "days")) && s[0]) {
        if (!(string_isnumber(s) && (atoi(s) >= 0))) {
            session_message(session,
                            "Days option must be non-negative integer");
            generate_form(session);
            return;

        }
        account->vacation_days = atoi(s);
    }

    if ((s = assoc_lookup(h, "aliases")) && s[0]) {
        string_strdup(&account->vacation_aliases, s);

        if (!(addr=addr_parse_destructive(s, ""))) {
            session_message(session, "Alias list invalid: %s", ml_errmsg());
            rc = NIL;
        } else {
            for (a = addr; a; a = a->next) {
                if (!(a->host && a->host[0])) {
                    session_message(session,
                                    "Unqualified address in alias list");
                    rc = NIL;
                    break;
                }
            }
            mail_free_address(&addr);
        }
    } else
        string_strdup(&account->vacation_aliases, "");

    if (rc && account_vacation_update(session->account, request->pool)
        && account_mail_update(session->account, request->pool)) {
        session_message(session, "Updated vacation message");
        session_redirect(session, request, "manage");
        return;
    }

    /* Report some kind of error */
    generate_form(session);
}

==> prayer-1.0.16.tar.gz: prayer-1.0.16/prayer/cmd_vacation.c <==
/* $Cambridge: hermes/src/prayer/prayer/cmd_vacation.c,v 1.10 2005/09/28 11:34:49 dpc22 Exp $ */
/************************************************
 *    Prayer - a Webmail Interface              *
 ************************************************/

/* Copyright (c) University of Cambridge 2000 - 2002 */
/* See the file NOTICE for conditions of use and distribution. */

#include "prayer_session.h"

static void generate_form(struct session *session)
{
    struct request *request = session->request;
    struct buffer *b = request->write_buffer;
    struct account *account = session->account;
    char *aliases = ((account->vacation_aliases)
                     ? (account->vacation_aliases) : NIL);
    char *days = string_itoa(request->pool, account->vacation_days);

    /* Title bar */
    html_start(session, b, "Vacation message");
    html_banner_toolbar(session, b, "vacation", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Vacation Message</h2>" CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "vacation");

    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    /* use_vacation */
    html_form_table_checkbox(b, "Enable Vacation Message.", "use_vacation",
                             account->vacation_enabled);

    if (session->sieved_server) {
        /* alt_addr */
        html_form_table_field(b, "&nbsp;&nbsp;&nbsp;Aliases:",
                              "aliases", aliases, 40);

        /* Days */
        html_form_table_field(b, "Interval (in days):", "days", days, 3);

    } else {
        /* alt_addr */
        html_form_table_field(b, "&nbsp;&nbsp;&nbsp;Aliases:",
                              "aliases", aliases, 50);
    }

    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);

    /* Start of vacation */
    bprintf(b,
            "<textarea name=\"vacation_msg\" rows=\"8\" cols=\"80\">"
            CRLF);
    if (account->vacation_msg && account->vacation_msg[0])
        html_quote_string(b, account->vacation_msg);
    bputs(b, "</textarea>" CRLF);

    /* Submit and cancel buttons */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    html_form_table_submit(b, "sub_apply", "Apply");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);

    /* End table */
    html_form_end(b);

    html_end(b);
    response_html(request, 200);        /* Success */
}

static void generate_error(struct session *session)
{
    struct request *request = session->request;
    struct account *account = session->account;
    char *msg = account_fetch_message(account);
    struct buffer *b = request->write_buffer;

    /* Title bar */
    html_start(session, b, "Vacation message");
    html_banner_toolbar(session, b, "vacation", 0);

    /* Title line */
    bputs(b, "<h2 align=\"center\">Vacation Messages</h2>" CRLF);

    bputs(b, "<b>Error:</b> ");
    if (msg) {
        html_quote_string(b, msg);
        session_log(session, "[cmd_vacation] %s", msg);
    } else {
        bputs(b, "Unable to check mail processing status with server");
    }
    bputs(b, "" CRLF);

    /* Form body */
    html_form_start(session, b, "POST", "vacation");

    /* Cancel button */
    bputs(b, "<table>" CRLF);
    bputs(b, "<tr>" CRLF);
    html_form_table_submit(b, "sub_cancel", "Back to Options Screen");
    bputs(b, "</tr>" CRLF);
    bputs(b, "</table>" CRLF);
    html_form_end(b);

    html_end(b);
    response_html(request, 200);
}

void cmd_vacation(struct session *session)
{
    struct request *request = session->request;
    struct assoc *h = NIL;
    struct account *account = session->account;
    ADDRESS *addr = NIL;
    ADDRESS *a = NIL;
    char *s;
    BOOL rc = T;

    if (request->method != POST) {
        if (!account_mail_check(account, request->pool))
            generate_error(session);
        else
            generate_form(session);
        return;
    }

    /* Form post */
    request_decode_post(request);
    h = request->form;

    if (!(s = assoc_lookup(h, "sub_apply"))) {
        session_redirect(session, request, "manage");   /* parent screen */
        return;
    }

    account->vacation_enabled =
        (assoc_lookup(h, "use_vacation")) ? T : NIL;

    if ((s = assoc_lookup(h, "vacation_msg")))
        string_strdup(&account->vacation_msg, s);
    else
        string_strdup(&account->vacation_msg, "");

    if ((s = assoc_lookup(h, "days")) && s[0]) {
        if (!(string_isnumber(s) && (atoi(s) >= 0))) {
            session_message(session,
                            "Days option must be non-negative integer");
            generate_form(session);
            return;

        }
        account->vacation_days = atoi(s);
    }

    if ((s = assoc_lookup(h, "aliases")) && s[0]) {
        string_strdup(&account->vacation_aliases, s);

        if (!(addr=addr_parse_destructive(s, ""))) {
            session_message(session, "Alias list invalid: %s", ml_errmsg());
            rc = NIL;
        } else {
            for (a = addr; a; a = a->next) {
                if (!(a->host && a->host[0])) {
                    session_message(session,
                                    "Unqualified address in alias list");
                    rc = NIL;
                    break;
                }
            }
            mail_free_address(&addr);
        }
    } else
        string_strdup(&account->vacation_aliases, "");

    if (rc) {
        if (account_vacation_update(account, request->pool) &&
            account_mail_update(account, request->pool)) {
            session_message(session, "Updated vacation message");
            session_redirect(session, request, "manage");
            return;
        } else {
            char *msg = account_fetch_message(account);

            session_message(session,
                            "Failed to update vacation message: %s", msg);
            session_log(session,
                        "[cmd_vacation] Failed to update vacation message: %s",
                        msg);
        }
    }

    /* Report some kind of error */
    generate_form(session);
}
